@@ -117,7 +117,7 @@ def read_image(file_name, resize_x=0, resize_y=0, rescale_pixel_intensity=False)
         String of bytes that can be used as the second argument in 
         experiment.publish_image()
     """
-    source = cv2.imread(file_name, cv2.IMREAD_ANYCOLOR | cv2.IMREAD_ANYDEPTH) 
+    source = cv2.imread(file_name)#, cv2.IMREAD_ANYCOLOR | cv2.IMREAD_ANYDEPTH) 
     if resize_x != 0 and resize_y != 0:
         source = cv2.resize(source, (resize_x, resize_y))
     if rescale_pixel_intensity:
@@ -224,6 +224,10 @@ class experiment:
         Password for the supplied MDML username
     host : str
         MDML host,  this will be given to you by an MDML admin
+    s3_access_key : str
+        S3 access key provided by an MDML admin
+    s3_secret_key : str
+        S3 secret key provided by an MDML admin
     
     """
 
@@ -270,6 +274,29 @@ class experiment:
             except:
                 print("ERROR! Connection to s3 failed.")
 
+    def s3_login(self, s3_access_key, s3_secret_key):
+        """
+        Create connection to BIS S3 object store for use in image streaming
+
+        Parameters
+        ==========
+        s3_access_key : str
+            S3 access key provided by an MDML admin
+        s3_secret_key : str
+            S3 secret key provided by an MDML admin
+        """
+        # Creating boto3 (s3) client connection
+        session = boto3.session.Session()
+        try:
+            self.s3_client = session.client(
+                service_name='s3',
+                aws_access_key_id=s3_access_key,
+                aws_secret_access_key=s3_secret_key,
+                endpoint_url='https://s3.it.anl.gov:18082'
+            )
+        except:
+            print("ERROR! Connection to s3 failed.")
+
     def globus_login(self):
         """
         Perform a Globus login to acquire auth tokens for FuncX analyses. Must be done before experiment.add_config()
@@ -501,8 +528,7 @@ class experiment:
         if self.tokens:
             payload['globus_token'] = self.tokens['funcx_service']['access_token']
         else:
-            print("No globus token found. You must use the .globus_login() method first.\
-                    This can be ignored if you are not using funcX for analysis.")
+            print("No globus token found. You must use the .globus_login() before starting an analysis.")
             return
 
         # Send data via MQTT
@@ -553,7 +579,7 @@ class experiment:
         # Send data via MQTT
         self.client.publish(topic, json.dumps(payload))
 
-    def publish_image_s3(self, device_id, img_filename, timestamp = 0, metadata = {}):
+    def publish_image_s3(self, device_id, img_filename, obj_name, timestamp = 0, metadata = {}):
         """
         Publish an image to MDML
 
@@ -563,10 +589,9 @@ class experiment:
             Unique string identifying the device that created the data.
             This must correspond with the experiment's configuration
         img_filename : str
-            filename to store the file in the MDML. Can only contain letters, 
-            numbers, underscores and must end with a valid file extension. 
-            If left blank, filenames will the experiment ID followed by an
-            index (e.g. EXPID_1.JPG, EXPID_2.JPG...)
+            filename where the image is stored
+        obj_name : str
+            A unique object name that will be used in the object store
         timestamp : int
             Unix time in nanoseconds. Can be supplied by the unix_time()
             function in this package
@@ -581,7 +606,9 @@ class experiment:
         # Base payload
         payload = {
             'filename': img_filename,
-            'data_type': 'image'
+            'obj_name': obj_name,
+            'data_type': 'image',
+            'send_method': 's3'
         }
         # Adding metadata if necessary
         payload['metadata'] = metadata
@@ -592,14 +619,14 @@ class experiment:
         #         return
         #     else:
         #         payload['filename'] = img_filename
-        payload['filename'] = img_filename
-        # Adding timing code
+
+        # Adding timestamp
         if timestamp == 0:
             timestamp = unix_time()
         payload['timestamp'] = timestamp
         payload = json.dumps(payload)
         # Upload to s3
-        self.s3_client.upload_file(img_filename, f'mdml-{self.experiment_id.lower()}', img_filename)
+        self.s3_client.upload_file(img_filename, f'mdml-{self.experiment_id.lower()}', obj_name)
         # Publish it
         self.client.publish(topic, payload)
         
@@ -640,13 +667,13 @@ class experiment:
         }
         # Adding metadata if necessary
         payload['metadata'] = metadata
-        # Check for valid filename
-        if filename != '':
-            if re.match(r"^[\w]+\.[A-Za-z0-9]+$", filename) == None:
-                print("Filename not valid. Can only contains letters, numbers, and underscores.")
-                return
-            else:
-                payload['filename'] = filename
+        # # Check for valid filename
+        # if filename != '':
+        #     if re.match(r"^[\w]+\.[A-Za-z0-9]+$", filename) == None:
+        #         print("Filename not valid. Can only contains letters, numbers, and underscores.")
+        #         return
+        #     else:
+        #         payload['filename'] = filename
         # Adding timing code
         if timestamp == 0:
             timestamp = unix_time()
@@ -796,7 +823,7 @@ class experiment:
             self.debugger.terminate()
         print("Debugger stopped.")
 
-    def replay_experiment(self, filename):
+    def replay_experiment(self, filename, speedup=1):
         """
         Replay an old experiment by specifying a tar file output from MDML
         
@@ -805,6 +832,8 @@ class experiment:
         ----------
         filename : str
             absolute filepath of the tar file for the experiment you would like to replay.
+        speedup : int
+            speedup of data rates
         """
         # Validate file
         try:
@@ -858,7 +887,8 @@ class experiment:
                                                         exp_dir, \
                                                         device_data_types[i], \
                                                         exp_start_time, \
-                                                        sim_start_time,))
+                                                        sim_start_time, \
+                                                        speedup))
                 tmp.setDaemon(True)
                 tmp.start()
             time.sleep(4)
@@ -867,7 +897,7 @@ class experiment:
             self.reset()
             return
 
-    def _replay_file(self, device_id, file_dir, data_type, exp_start_time, sim_start_time):
+    def _replay_file(self, device_id, file_dir, data_type, exp_start_time, sim_start_time, speedup):
         with open(file_dir + '/' + device_id) as data_file:
             _ = data_file.readline()
             data = data_file.readlines()
@@ -875,7 +905,7 @@ class experiment:
         while True:
             # Get next timestamp TODO delimiter needs to come from experiment configuration file
             next_dat_time = int(re.split('\t', data[0])[0])
-            exp_delta = next_dat_time - exp_start_time
+            exp_delta = float(next_dat_time - exp_start_time)/speedup
             sim_delta = unix_time(ret_int=True) - sim_start_time
 
             if (sim_delta >= exp_delta):
@@ -888,7 +918,7 @@ class experiment:
                 elif data_type == "image":
                     img_filename = file_dir + '/' + re.split('\t', data[0])[1]
                     img_byte_string = read_image(img_filename)
-                    self.publish_image(device_id, img_byte_string, timestamp=sim_start_time + exp_delta)
+                    self.publish_image(device_id, img_byte_string, timestamp=sim_start_time + sim_delta)
                 else:
                     print("DATA_TYPE IN CONFIGURATION NOT SUPPORTED")
                 del data[0]
